#!/usr/bin/env bash
#
# Description: grepの違いによる正規表現の違いを確認する
#              与えるデータの妥当性を確認する
#   制限事項:このようにオプション引数をスペースで分けると、オプションの引き渡しが失敗する
#   ./tregex.bash -_ n -m 5

OS=$(uname -a)
COLOROPT="--color=auto"
TC="tregex.d"

usage_exit() {
    printf "%s [-?] [-_TESTCASE] [OPTIONS] <regex> ...\n" $0
    echo "   -_ : 登録してあるテストケースを実行"
    _pcre && [ -d "${0%/*}/${TC}/" ] && $GREP -oh '(?<=TESTCASES=).*' "${0%/*}/${TC}/"* 
    echo
    echo "  変数GREPに実行するgrepを指定できる"
    echo "  $ GREP=\"ggrep -P\" $0"
    exit
}

_pcre() { # PCREを用いる, GREPが設定されている場合は何もしない
    [ -n "$GREP" ] && return 0
    case "${OS}" in
        Darwin*)
            if [ -n "$(which ggrep)" ]; then
                GREP="ggrep -P"
            elif [ -x "/usr/local/bin/pcre2grep" ]; then
                GREP="pcre2grep"
            elif [ -x "/usr/local/bin/pcregrep" ]; then
                GREP="pcregrep"
            fi
            ;;
        *)      GREP="grep -P" ;;
    esac
    if ! $GREP '' <<<"" > /dev/null 2>&1  ;then
        echo "$GREP has no pcre"
        return 1
    fi
    return 0
}

while getopts :P?_: OPT ; do
  case $OPT in
    P) _pcre ;;
    _) TEST=${OPTARG} ;;
    *)
        [ "$OPT" = '?' -a "${OPTARG}" = '?' ] &&  usage_exit
        OPTIONS="${OPTIONS}${OPTARG}" ;;
  esac
done
shift $((OPTIND - 1))

[ -n "${OPTIONS}" ] && OPTIONS="-${OPTIONS}"

#echo "test case = $TEST"
if [ -n "$TEST" ]; then
    # テストケースは、$0と同じディレクトリの${TC}ディレクトリ内に配置
    #echo "${0%/*}/${TC}/"
    if [ -d "${0%/*}/${TC}/" ]; then
        for f in "${0%/*}/${TC}/"* ; do source "$f" ; done
        echo "Tried testcase '$TEST'."
        exit
    fi
fi

#set -x
${GREP-egrep} ${COLOROPT} ${OPTIONS} "$@"
echo "Done."
