##### Password Validation
# SD202204 UNIXテキスト処理の達人
# TESTCASES=[pw8]         パスワードのバリデーション: 8文字以上, 1文字以上ぞれぞれ数字,小文字,大文字,特殊文字を含むこと
# TESTCASES=              補題:パスワードバリデーション 補題: 5文字以上で、数字と小文字を1文字以上含むこと
# TESTCASES=[pwlere]            EREで解決
# TESTCASES=[pwlng1]            ERE 全然だめなケース
# TESTCASES=[pwlng2]            ERE だめなケース2
# TESTCASES=[pwldemorgan]       ERE ド・モルガンを用いた解決
# TESTCASES=[pwlmulti]          ERE 複数のgrepをフィルタで連結して解決
# TESTCASES=[pwlng3]            ERE 駄目なケース3 -e オプションを複数指定してみる
# TESTCASES=[pwlpcre]           PCREによる解決

TESTPW='12345
abcde
ABCDE6
ABCDEa
ab12
Abcdef
......0.
.....s..
........
s...s
ここから下がOK
t.0..
0.t..
t0...
0t...
t...0
0...t
.0.t.
.t.0.
..0.t
..t.0
...0t
...t0
thisis0k'

lemma() {
    echo "SD202204 UNIXテキスト処理の達人"
    echo  "5文字以上で、数字と小文字を1文字以上含むこと";
}

case "$TEST" in
    pw8) _pcre
        echo "SD202204 UNIXテキスト処理の達人"
        echo "パスワードのバリデーション: $GREP"
        echo "8文字以上, 1文字以上ぞれぞれ数字,小文字,大文字,特殊文字を含むこと"
        regexp='^(?=.*?[A-Z])(?=.*?[a-z])(?=.*?[0-9])(?=.*?[^\w\s]).{8,}$'
        echo "regexp='$regexp'"
        $GREP ${COLOROPT} ${OPTIONS} ${regexp} <<"TESTDATA"
12345678
abcdefgh
1a345678
it3Pw.
ThisisPassword
ThisisPassword
This0isPassword
ThisisPassword.
ThisIsPassw0rd.
TESTDATA
        exit
        ;; # TESTDATA最後の1行のみが条件を満たす

    pwlere) echo "パスワードのバリデーション: ERE"
        lemma
        regexp='[a-z][0-9].{3}|[0-9][a-z].{3}|[a-z].[0-9].{2}|[0-9].[a-z].{2}|.[a-z][0-9].{2}|.[0-9][a-z].{2}|[a-z].{2}[0-9].|[0-9].{2}[a-z].|.[a-z].[0-9].|.[0-9].[a-z].|.{2}[a-z][0-9].|.{2}[0-9][a-z].|[a-z].{3,}[0-9]|[0-9].{3,}[a-z]|.{2}[a-z].[0-9]|.{2}[0-9].[a-z]|.{3}[a-z][0-9]|.{3}[0-9][a-z]'
        echo "regexp='$regexp'"
        ${GREP-egrep} ${COLOROPT} ${OPTIONS} ${regexp} <<<"${TESTPW}"
        exit
        ;;

    pwlng1) echo "パスワードバリデーション: ERE NG1 全然だめ"
        lemma
        regexp='([a-z].[0-9]){5,}|([0-9].[a-z]){5,}'
        echo "regexp='$regexp'"
        ${GREP-egrep} ${COLOROPT} ${OPTIONS} ${regexp} <<<"${TESTPW}"
        exit
        ;;

    pwlng2) echo "パスワードバリデーション: ERE NG2"
        lemma
        regexp='[a-z].{3}[0-9]|[0-9].{3}[a-z]'
        echo "regexp='$regexp'"
        ${GREP-egrep} ${COLOROPT} ${OPTIONS} ${regexp} <<<"${TESTPW}"
        # t...0
        # 0...t
        # thisis0k
        exit
        ;;

    pwldemorgan) echo "パスワードバリデーション: ERE ド・モルガンを使う"
        lemma
        echo " a. 5文字以上"
        echo " b. 数字を1文字以上含んでいる"
        echo " c. 小文字を1文字以上含んでいる"
        echo " a. ∧ b. ∧ c."
        echo

        regexp='^.{0,4}$' 
        echo '¬(a). 任意の0〜4文字の文字列: ' "$regexp"
        ${GREP-egrep} ${COLOROPT} ${OPTIONS} ${regexp} <<<"${TESTPW}"
        echo

        regexp='^[^0-9]*$'
        echo '¬(b). 数字を含まない0文字以上の文字列: ' "regexp"
        ${GREP-egrep} ${COLOROPT} ${OPTIONS} ${regexp} <<<"${TESTPW}"
        echo

        echo "l1. 補題:任意の0〜4文字の文字列 もしくは 数字を含まない0文字以上の文字列"
        regexp='^(.{0,4}|[^0-9]*)$'
        echo '¬(a) ∪ ¬(b): ' "$regexp"
        ${GREP-egrep} ${COLOROPT} ${OPTIONS} ${regexp} <<<"${TESTPW}"
        echo

        regexp='^[^a-z]*$'
        echo '¬(c). a-zを含まない0文字以上の文字列: ' "${regexp}"
        ${GREP-egrep} ${COLOROPT} ${OPTIONS} ${regexp} <<<"${TESTPW}"
        echo

        regexp='^(.{0,4}|[^0-9]*|[^a-z]*)$'
        echo 'l2. ¬(a) ∪ ¬(b) ∪ ¬(c) : ' "${regexp}"
        ${GREP-egrep} ${COLOROPT} ${OPTIONS} ${regexp} <<<"${TESTPW}"
        echo

        echo "¬( ¬(a) ∪ ¬(b) ∪ ¬(c)) ≡ a ∩ b ∩ c"
        regexp='^(.{0,4}|[^0-9]*|[^a-z]*)$'
        echo "$regexp"
        ${GREP-egrep} ${COLOROPT} ${OPTIONS} ${regexp} <<<"${TESTPW}"
        echo
        exit
        ;;

    pwlmulti) echo "パスワードバリデーション: 多重処理1."
        lemma
        echo "egrep '.{5,}' | egrep '[0-9]+' | egrep '[a-z]+'"
        ${GREP-egrep} '.{5,}' <<<"${TESTPW}" | ${GREP-egrep} '[0-9]+' | ${GREP-egrep} '[a-z]+'
        echo
        exit
        ;;

    pwlng3) echo "パスワードバリデーション: NG 多重処理"
        lemma
        echo "egrep -e '.{5,}' -e '[0-9]+' -e '[a-z]+'"
        ${GREP-egrep} -e '.{5,}' -e '[0-9]+' -e '[a-z]+' <<<"${TESTPW}" 
        echo
        exit
        ;;

    pwlpcre) _pcre
        echo "パスワードバリデーション: $GREP"
        lemma
        regexp='^(?=.*?[a-z])(?=.*?[0-9]).{5,}$'
        echo 'PCREでやれば簡単: ' "$regexp"
        ${GREP-egrep} ${COLOROPT} ${OPTIONS} ${regexp} <<<"${TESTPW}"
        echo
        exit
        ;;
esac

unset lemma
unset TESTPW
